#!/bin/bash
set -e

function usage() {
    echo -e "
USAGE:
    $(basename $0) --help                       # 使用帮助
    $(basename $0) --mirror http://apt.site     # 源地址
    $(basename $0) --suite v101|10.1|10.0       # 源suite
    $(basename $0) --name rootfs_name           # 名称
    $(basename $0) --arch amd64|arm64           # 架构
    $(basename $0) --include 'kylin-keyring'    # v10额外装的包
    $(basename $0) --conf config.sh             # 配置文件
    $(basename $0) --tasks 'task1,task2'        # 任务
      taskname:
        rootfs          制作rootfs
        config          配置rootfs
        config_openbox  配置openbox
        tar             打包rootfs
        ext4            制作ext4
        iso             制作iso

EXAMPLE:
    $(basename $0) --arch amd64 --tasks rootfs
    $(basename $0) --arch amd64 --tasks config
    $(basename $0) --arch amd64 --tasks tar
    $(basename $0) --arch amd64 --tasks ext4
    $(basename $0) --arch amd64 --tasks 'rootfs,config,tar'

制作rootfs:
    $(basename $0) --arch arm64 --name dianwang --conf config-dianwang.sh --tasks 'rootfs,config,tar'
    $(basename $0) --arch arm64 --name nanrui   --conf config-nanrui.sh   --tasks 'rootfs,config,tar'

制作openbox:
    $(basename $0) --arch amd64 --name openbox  --conf config-openbox.sh --tasks 'rootfs,config,config_openbox,tar'

制作ukui:
    $(basename $0) --arch arm64 --name ukui --mirror http://172.29.157.20/mirror/juniper --suite v100 --conf config-ukui-10.0.sh --include 'kylin-keyring' --tasks 'rootfs,config,config_linbo,tar,ext4'
    $(basename $0) --arch arm64 --name ukui --conf config-ukui-10.1.sh --tasks 'rootfs,config,config_linbo,tar,ext4'

制作iso:
    $(basename $0) --arch amd64 --name iso --conf config-iso.sh --tasks 'rootfs,config,config_openbox,iso'

制作wic:
  ubuntu:
    $(basename $0) --arch amd64 --name ubuntu --mirror http://172.29.157.20/ubuntu --suite focal --conf config-ubuntu.sh --tasks 'rootfs,config,tar,wic'
    $(basename $0) --arch arm64 --name ubuntu --mirror http://172.29.157.20/ubuntu-ports --suite focal --conf config-ubuntu.sh --tasks 'rootfs,config,tar,wic'
  kylin:
    $(basename $0) --arch arm64 --name kylin --conf config-ubuntu.sh --tasks 'rootfs,config,tar,wic'
    $(basename $0) --arch arm64 --name openbox-kysec --conf config-openbox-kysec.sh --tasks 'rootfs,config,config_openbox,tar,wic'
    "

    exit 1
}

# 判断参数个数
if [ $# -eq 0 ]; then
    usage
fi

trap "exit_function" 0

: 自定义时间戳 <<BLOCK
说明:
  脚本默认取当前日期作为目录名的一部分
  如果第二天想接着前几天做的继续修改，可以先设置时间戳

export TIMESTAMP="20220714"
BLOCK

timestamp=${TIMESTAMP:-$(date "+%Y%m%d")}

CURRENT_DIR=$(dirname $(readlink -f $0))

function do_rootfs() {
    # 先删除 rootfs 目录
    if [ -d "${ROOTFS_DIR}" ]; then
        echo "sudo rm -rf ${ROOTFS_DIR}"
        sudo rm -rf ${ROOTFS_DIR}
    fi

    # 制作其他架构，判断是否安装 binfmt-support,qemu-user-static
    if [ "$(dpkg --print-architecture)" != "${ARCH}" ]; then
        if ! dpkg -l | grep binfmt-support; then
            sudo apt install -y binfmt-support
        fi
        if ! dpkg -l | grep qemu-user-static; then
            sudo apt install -y qemu-user-static
        fi
    fi

    # 制作 rootfs
    if [ -n "${INCLUDES}" ]; then
        sudo debootstrap --no-check-gpg --variant=minbase --arch=${ARCH} --include=${INCLUDES} --components=main ${SUITE} ${ROOTFS_DIR} ${MIRROR} gutsy
    else
        sudo debootstrap --no-check-gpg --variant=minbase --arch=${ARCH} --components=main ${SUITE} ${ROOTFS_DIR} ${MIRROR} gutsy
    fi
}

function do_config() {
    source ${ROOTFS_ENV_FILE}

    # 宿主机挂载 设备节点 到 rootfs
    sudo mount --bind /dev ${ROOTFS_DIR}/dev
    sudo mount --bind /run ${ROOTFS_DIR}/run
    sudo mount -t devpts devpts ${ROOTFS_DIR}/dev/pts
    sudo mount -t proc proc ${ROOTFS_DIR}/proc
    sudo mount -t sysfs sysfs ${ROOTFS_DIR}/sys

    # 配置 rootfs
    CONFFILE=${CONF:-config-default.sh}
    if [ -f ${CURRENT_DIR}/${CONFFILE} ]; then
        sudo cp ${CURRENT_DIR}/${CONFFILE} ${ROOTFS_DIR}
        sudo chroot ${ROOTFS_DIR} bash /${CONFFILE}
        sudo rm ${ROOTFS_DIR}/${CONFFILE}
    else
        echo "${CONFFILE} not exits!"
    fi

    # 卸载宿主设备节点
    sudo umount ${ROOTFS_DIR}/dev/pts
    sudo umount ${ROOTFS_DIR}/dev
    sudo umount ${ROOTFS_DIR}/run
    sudo umount ${ROOTFS_DIR}/sys
    sudo umount ${ROOTFS_DIR}/proc
}

# openbox相关配置
function do_config_openbox() {
    if [ -d ${CURRENT_DIR}/etc/xdg/openbox ]; then
        sudo cp -v -rf -a ${CURRENT_DIR}/etc/xdg/openbox ${ROOTFS_DIR}/etc/xdg
        sudo chmod 755 -R ${ROOTFS_DIR}/etc/xdg/openbox
    fi
}

# 林博 rootfs 需要 fstab 文件
function do_config_linbo() {
    sudo cp -v ${CURRENT_DIR}/etc/fstab ${ROOTFS_DIR}/etc
}

# 打包 rootfs
function do_tar() {
    source ${ROOTFS_ENV_FILE}
    echo "sudo tar zcf ${ROOTFS_DIR}.tar.gz -C ${ROOTFS_DIR} ."
    sudo tar zcvf ${ROOTFS_DIR}.tar.gz -C ${ROOTFS_DIR} .
    md5sum ${ROOTFS_DIR}.tar.gz | awk '{print $1}' >${ROOTFS_DIR}.tar.gz.md5
}

# 制作 ext4
function do_ext4() {
    source ${ROOTFS_ENV_FILE}

    # 计算rootfs目录大小，默认在原有大小基础上加 600M
    dusize=$(sudo du -s ${ROOTFS_DIR} | awk '{print $1}')
    size=$(echo "(${dusize} / 1024) + 600" | bc)

    # size=8192   # 指定 8G 大小
    size=4096 # 指定 4G 大小

    dd if=/dev/zero of=${ROOTFS_DIR}.ext4 bs=1M count=${size} status=progress
    sudo mkdir -p ./ext4_mount
    yes | mkfs.ext4 ${ROOTFS_DIR}.ext4
    sudo mount ${ROOTFS_DIR}.ext4 ./ext4_mount
    sudo cp -rf -a ${ROOTFS_DIR}/* ./ext4_mount
    sudo umount ./ext4_mount
    sudo rmdir ./ext4_mount
    echo "tar zcvf ${ROOTFS_DIR}.ext4.tar.gz ${ROOTFS_DIR}.ext4"
    tar zcf ${ROOTFS_DIR}.ext4.tar.gz ${ROOTFS_DIR}.ext4
    md5sum ${ROOTFS_DIR}.ext4.tar.gz | awk '{print $1}' >${ROOTFS_DIR}.ext4.tar.gz.md5
}

function do_iso() {
    source ${ROOTFS_ENV_FILE}

    WORK_DIR=$(pwd)
    CDROM="cdrom"
    # create cdrom directory
    mkdir -pv ${CDROM}/{casper,isolinux}

    # 拷贝 ky-installer.cfg 和 .kylin-post-actions
    cp -v ${CURRENT_DIR}/iso/ky-installer.cfg ${CDROM}
    cp -v ${CURRENT_DIR}/iso/.kylin-post-actions ${CDROM}

    # copy kernel
    sudo cp -L -v ${ROOTFS_DIR}/boot/vmlinuz ${CDROM}/casper/vmlinuz
    sudo cp -L -v ${ROOTFS_DIR}/boot/initrd.img ${CDROM}/casper/initrd

    touch ${CDROM}/kylin
    # GRUB menu
    cat >${CDROM}/isolinux/grub.cfg <<EOF
search --set=root --file /kylin
insmod all_video

set default=0
set timeout=10

menuentry "Try KylinOS" {
    linux /casper/vmlinuz boot=casper nopersistent toram quiet splash ---
    initrd /casper/initrd
}

menuentry "Install KylinOS" {
    linux /casper/vmlinuz boot=casper only-ubiquity quiet splash ---
    initrd /casper/initrd
}

menuentry "Auto Install KylinOS" {
    linux /casper/vmlinuz boot=casper only-ubiquity quiet splash automatic ---
    initrd /casper/initrd
}

# menuentry "Check disc" {
#     linux /casper/vmlinuz boot=casper integrity-check quiet splash ---
#     initrd /casper/initrd
# }
EOF

    # # create manifest
    # sudo chroot ${ROOTFS_DIR} dpkg-query -W --showformat='${Package} ${Version}\n' >${CDROM}/casper/filesystem.manifest
    # cp -v ${CDROM}/casper/filesystem.manifest ${CDROM}/casper/filesystem.manifest-desktop
    # sed -i '/ubiquity/d' ${CDROM}/casper/filesystem.manifest-desktop
    # sed -i '/casper/d' ${CDROM}/casper/filesystem.manifest-desktop
    # sed -i '/discover/d' ${CDROM}/casper/filesystem.manifest-desktop
    # sed -i '/laptop-detect/d' ${CDROM}/casper/filesystem.manifest-desktop
    # sed -i '/os-prober/d' ${CDROM}/casper/filesystem.manifest-desktop

    # create squashfs
    sudo rm -rf ${CDROM}/casper/filesystem.squashfs
    sudo mksquashfs ${ROOTFS_DIR} ${CDROM}/casper/filesystem.squashfs
    printf $(sudo du -sx --block-size=1 ${ROOTFS_DIR} | cut -f1) >${CDROM}/casper/filesystem.size

    cd ${WORK_DIR}/${CDROM}

    grub-mkstandalone \
        --format=x86_64-efi \
        --output=isolinux/bootx64.efi \
        --locales="" \
        --fonts="" \
        "boot/grub/grub.cfg=isolinux/grub.cfg"

    (
        cd isolinux &&
            dd if=/dev/zero of=efiboot.img bs=1M count=10 &&
            sudo mkfs.vfat efiboot.img &&
            LC_CTYPE=C mmd -i efiboot.img efi efi/boot &&
            LC_CTYPE=C mcopy -i efiboot.img ./bootx64.efi ::efi/boot/
    )

    grub-mkstandalone \
        --format=i386-pc \
        --output=isolinux/core.img \
        --install-modules="linux16 linux normal iso9660 biosdisk memdisk search tar ls" \
        --modules="linux16 linux normal iso9660 biosdisk search" \
        --locales="" \
        --fonts="" \
        "boot/grub/grub.cfg=isolinux/grub.cfg"
    cat /usr/lib/grub/i386-pc/cdboot.img isolinux/core.img >isolinux/bios.img

    sudo /bin/bash -c "(find . -type f -print0 | xargs -0 md5sum | grep -v -e 'md5sum.txt' -e 'bios.img' -e 'efiboot.img' > md5sum.txt)"

    sudo xorriso \
        -as mkisofs \
        -iso-level 3 \
        -allow-lowercase \
        -full-iso9660-filenames \
        -volid "Kylin V10 SP1 Test" \
        -output "../kylin-${timestamp}.iso" \
        -eltorito-boot boot/grub/bios.img \
        -no-emul-boot \
        -boot-load-size 4 \
        -boot-info-table \
        --eltorito-catalog boot/grub/boot.cat \
        --grub2-boot-info \
        --grub2-mbr /usr/lib/grub/i386-pc/boot_hybrid.img \
        -eltorito-alt-boot \
        -e EFI/efiboot.img \
        -no-emul-boot \
        -append_partition 2 0xef isolinux/efiboot.img \
        -m "isolinux/efiboot.img" \
        -m "isolinux/bios.img" \
        -graft-points \
        "/EFI/efiboot.img=isolinux/efiboot.img" \
        "/boot/grub/bios.img=isolinux/bios.img" \
        "."
}

# 制作磁盘镜像
function do_wic() {
    # 磁盘文件名
    IMAGE_FILE="${ROOTFS_DIR}.wic"

    # 计算rootfs目录大小，默认在原有大小基础上加 600M
    dusize=$(sudo du -s ${ROOTFS_DIR} | awk '{print $1}')
    size=$(echo "(${dusize} / 1024) + 600" | bc)

    dd if=/dev/zero of="${IMAGE_FILE}" bs=1M count=${size} status=progress

    # 创建两个分区，一个用来存放kernel、initrd；另一个存放根文件系统
    sgdisk -n 0:0:+128M -c 0:boot "${IMAGE_FILE}"
    sgdisk -n 0:0:0 -c 0:rootfs "${IMAGE_FILE}"
    sgdisk -p "${IMAGE_FILE}"

    disk_path=$(readlink -f "${IMAGE_FILE}")
    LOOP_DEV=$(losetup -l -O NAME,BACK-FILE | grep "${disk_path}" | awk '{print $1}')

    if [ -z "${LOOP_DEV}" ]; then
        LOOP_DEV=$(losetup -f)
        # 将镜像关联到loop设备上
        sudo losetup -P ${LOOP_DEV} "${IMAGE_FILE}"
        # sudo partprobe ${LOOP_DEV}
    fi
    echo ${LOOP_DEV}

    # 格式化分区
    yes | sudo mkfs.fat ${LOOP_DEV}p1
    yes | sudo mkfs.ext4 ${LOOP_DEV}p2

    boot_uuid=$(sudo blkid -s UUID -o value ${LOOP_DEV}p1)
    root_uuid=$(sudo blkid -s UUID -o value ${LOOP_DEV}p2)
    part_uuid=$(sudo blkid -s PARTUUID -o value ${LOOP_DEV}p2)

    echo boot_uuid=${boot_uuid}
    echo root_uuid=${root_uuid}
    echo part_uuid=${part_uuid}

    # 拷贝文件
    mkdir -p {p1,p2}
    sudo mount ${LOOP_DEV}p1 ./p1/
    sudo mount ${LOOP_DEV}p2 ./p2/

    # copy efi to p1
    sudo cp -rf ${CURRENT_DIR}/${ARCH}/EFI/ ./p1/

    # copy kernel initrd
    (
        # 加载 gzio.mod 模块即可支持gz压缩格式的kernel
        sudo cp -L -v ${ROOTFS_DIR}/boot/vmlinuz ./p1/
        # # 解压gzip格式的vmlinux
        # if sudo file ./p1/vmlinuz | grep -q "gzip compressed data"; then
        #     sudo mv -v ./p1/vmlinuz{,.gz}
        #     sudo gunzip -v ./p1/vmlinuz.gz
        # fi
        sudo cp -L -v ${ROOTFS_DIR}/boot/initrd.img ./p1/

        # copy rootfs to p2
        sudo cp -rf -a ${ROOTFS_DIR}/* ./p2/
    )

    # generate grub
    echo "---> generate grub"
    sudo tee ./p1/EFI/BOOT/grub.cfg <<EOF
serial --unit=0 --speed=115200 --word=8 --parity=no --stop=1
default=myboot_uuid
timeout=5

menuentry 'myboot_uuid' {
    insmod gzio
    search --no-floppy --fs-uuid --set=root ${root_uuid}
    linux  /boot/vmlinuz root=UUID=${root_uuid} rootwait rootfstype=ext4 console=ttyS0,115200 console=tty0 quiet splash
    initrd /boot/initrd.img
}

menuentry 'myboot_partuuid' {
    linux /vmlinuz root=PARTUUID=${part_uuid} rootwait rootfstype=ext4 console=ttyS0,115200 console=tty0 quiet
    initrd /initrd.img
}

EOF

    # generate fstab
    echo "---> generate fstab"
    sudo mkdir -p ./p2/boot/efi
    sudo tee ./p2/etc/fstab <<EOF
UUID=${boot_uuid}   /boot/efi   vfat    rw,relatime 0 0
UUID=${root_uuid}   /           ext4    rw,relatime 0 1
EOF

    sudo umount ./p1 ./p2
    sudo losetup -d ${LOOP_DEV}

    tar zcvf ${IMAGE_FILE}.tar.gz ${IMAGE_FILE}
    md5sum ${IMAGE_FILE}.tar.gz | awk '{print $1}' >${IMAGE_FILE}.tar.gz.md5
}

function action_umount() {
    if mountpoint -q $1; then
        sudo umount $1
    fi
}

ARGS=$(getopt -a -o h -l help,name:,arch:,conf:,include:,tasks:,suite:,mirror: -- "$@")
eval set -- "${ARGS}"

while true; do
    case "$1" in
    -h | --help)
        usage
        shift
        ;;
    --mirror)
        MIRROR=$2
        shift
        ;;
    --suite)
        SUITE=$2
        shift
        ;;
    --name)
        NAME=$2
        shift
        ;;
    --arch)
        case "$2" in
        "amd64")
            ARCH=$2
            ;;
        "arm64")
            ARCH=$2
            ;;
        *)
            echo "not support this 'ARCH: $2'"
            exit
            ;;
        esac
        shift
        ;;
    --conf)
        CONF=$2
        shift
        ;;
    --include)
        INCLUDES=$2
        shift
        ;;
    --tasks)
        TASKS=$2
        shift
        ;;
    --)
        break
        ;;
    esac
    shift
done

if [ -z "${TASKS}" ]; then
    echo "no tasks"
    usage
fi

ROOTFS_DIR="${NAME:-rootfs}_${ARCH}-${timestamp}"
ROOTFS_ENV_FILE="${ROOTFS_DIR}.env"
MIRROR="${MIRROR:-http://172.29.157.20/my_mirror/kylin}"
SUITE=${SUITE:-v101}

# 保存变量，供其他步骤使用
(
    echo "ROOTFS_DIR=${ROOTFS_DIR}"
    echo "MIRROR=${MIRROR}"
    echo "SUITE=${SUITE}"
    echo "NAME=${NAME}"
    echo "ARCH=${ARCH}"
    echo "CONF=${CONF}"
    echo "TASKS=${TASKS}"
) | tee ${ROOTFS_ENV_FILE}

LOG_FILE=${ROOTFS_DIR}.log
# echo "running task: '${TASKS}', use 'tail -f ${LOG_FILE}' see detail log"
echo "running task: '${TASKS}'"

for task in $(echo ${TASKS} | tr ',' ' '); do
    echo "---> do_${task}"
    do_${task}
done | tee ${LOG_FILE} 2>&1
# done >${LOG_FILE} 2>&1

exit_function() {
    action_umount ${ROOTFS_DIR}/dev/pts
    action_umount ${ROOTFS_DIR}/dev
    action_umount ${ROOTFS_DIR}/run
    action_umount ${ROOTFS_DIR}/sys
    action_umount ${ROOTFS_DIR}/proc

    action_umount ./p1
    action_umount ./p2
    echo "exit_function"
}
