#!/bin/bash

: 制作磁盘镜像 <<BLOCK
tar xvf system.img.tar.gz -O | sudo dd of=/dev/sda status=progress
BLOCK

function usage() {
    echo "USAGE:
    $(basename $0) /path/rootfs
    "
    exit 1
}

# 判断参数个数
if [ $# -eq 0 ]; then
    usage
fi

# rootfs所在位置
ROOTFS_DIR=$1

# 磁盘文件名
IMAGE_FILE="system.img"

# 创建磁盘文件
function create_disk() {
    # 计算rootfs目录大小，默认在原有大小基础上加 600M
    dusize=$(sudo du -s ${ROOTFS_DIR} | awk '{print $1}')
    size=$(echo "(${dusize} / 1024) + 600" | bc)

    dd if=/dev/zero of="${IMAGE_FILE}" bs=1M count=2048 status=progress
}

# 创建两个分区，一个用来存放kernel、initrd；另一个存放根文件系统
function create_partition() {
    sgdisk -n 0:0:+100M -c 0:boot "${IMAGE_FILE}"
    sgdisk -n 0:0:0 -c 0:rootfs "${IMAGE_FILE}"
}

# 查看分区
function list_partition() {
    sgdisk -p "${IMAGE_FILE}"
}

# 寻找一个空闲的loop设备，并关联磁盘镜像文件
function find_loop_and_setup() {
    disk_path=$(readlink -f "${IMAGE_FILE}")
    LOOP_DEV=$(losetup -l -O NAME,BACK-FILE | grep "${disk_path}" | awk '{print $1}')

    if [ -z "${LOOP_DEV}" ]; then
        LOOP_DEV=$(losetup -f)
        # 将镜像关联到loop设备上
        sudo losetup ${LOOP_DEV} "${IMAGE_FILE}"
        sudo partprobe ${LOOP_DEV}
    fi
    echo ${LOOP_DEV}
}

# 格式化分区
function format_partition() {
    yes | sudo mkfs.fat ${LOOP_DEV}p1
    yes | sudo mkfs.ext4 ${LOOP_DEV}p2
}

function get_boot_uuid() {
    boot_uuid=$(sudo blkid -s UUID -o value ${LOOP_DEV}p1)
    echo ${boot_uuid}
}
function get_root_uuid() {
    root_uuid=$(sudo blkid -s UUID -o value ${LOOP_DEV}p2)
    echo ${root_uuid}
}

function get_part_uuid() {
    part_uuid=$(sudo blkid -s PARTUUID -o value ${LOOP_DEV}p2)
    echo ${part_uuid}
}

# 拷贝文件
function copy_files() {
    mkdir -p {p1,p2}
    sudo mount ${LOOP_DEV}p1 ./p1/
    sudo mount ${LOOP_DEV}p2 ./p2/

    # p1
    sudo cp -rf ./EFI/ p1
    # p2
    sudo cp -rf -a ~/temp/x86/ubuntu_amd64-20220714/* ~/p2

    sudo umount ./p1 ./p2
}

# 释放loop设备
function free_loopdev() {
    sudo losetup -d ${LOOP_DEV}
}

function gen_grub() {
    sudo tee ./p1/EFI/BOOT/grub.cfg <<EOF
serial --unit=0 --speed=115200 --word=8 --parity=no --stop=1
default=myboot
timeout=5

menuentry 'myboot'{
linux /vmlinuz root=UUID=${root_uuid} rootwait rootfstype=ext4 console=ttyS0,115200 console=tty0 quiet
initrd /initrd.img
}

menuentry 'myboot 2'{
linux /vmlinuz root=PARTUUID=${part_uuid} rootwait rootfstype=ext4 console=ttyS0,115200 console=tty0 quiet
initrd /initrd.img
}
EOF
}

function gen_fstab() {
    sudo mkdir -p ./p2/boot/efi
    sudo tee ./p2/etc/fstab <<EOF
UUID=${boot_uuid} /boot/efi vfat rw,relatime,errors=remount-ro  0 0
UUID=${root_uuid}   /   ext4    rw,relatime 0 1
EOF
}

create_disk
create_partition
# find_loop_and_setup
# format_partition

# free_loopdev
